package com.itko.lisa.ext.util;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;

/**
 * <pre>
 * -Djdbc.drivers=com.itko.lisa.vse.jdbc.driver.Driver
 * -Dlisa.jdbc.sim.require.remote=true
 * -Dlisa.jdbc.sim.port=2999
 * </pre>
 * 
 * @author <a href="mailto:luther@itko.com">Luther E. Birdzell</a>
 */
public class JdbcHelper
{
    private static Logger       log                  = Logger.getLogger(JdbcHelper.class);
    private static final String LISA_VSE_JDBC_DRIVER = "com.itko.lisa.vse.jdbc.driver.Driver";

    /**
     * 
     * 
     * @param url
     * @param driver
     * @param user
     * @param password
     * @return
     * @throws SQLException
     */
    public static Connection getVseConnection(String url,
                                              String driver,
                                              String user,
                                              String password)
        throws SQLException
    {
        Connection conn = null;

        try
        {
            try
            {
                Class.forName(driver);

                // use this instead of
                // -Djdbc.drivers=com.itko.lisa.vse.jdbc.driver.Driver
                Class.forName(LISA_VSE_JDBC_DRIVER);
            }
            catch ( ClassNotFoundException e )
            {
                log.error(e.getMessage(), e);
                throw new SQLException(e.getMessage());
            }

            String wrappedUrl = "jdbc:lisasim:driver=" + driver + ";url=" + url;

            log.info("Attempting to connect via LisaDriver; " + wrappedUrl
                    + " user=" + user);

            // Enumeration < Driver > drivers = DriverManager.getDrivers();
            // while ( drivers.hasMoreElements() )
            // {
            // Driver d = drivers.nextElement();
            // log.debug(d.getClass().getName());
            // }

            conn = DriverManager.getConnection(url, user, password);
        }
        catch ( Exception e )
        {
            log.error(e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return conn;
    }

    /**
     * 
     * 
     * @param url
     * @param driver
     * @param user
     * @param password
     * @return
     * @throws SQLException
     */
    public static Connection getLiveConnection(String url,
                                               String driver,
                                               String user,
                                               String password)
        throws SQLException
    {
        Connection conn = null;

        try
        {
            try
            {
                Class.forName(driver);
            }
            catch ( ClassNotFoundException e )
            {
                log.error(e.getMessage(), e);
                throw new SQLException(e.getMessage());
            }

            log.info("Attempting to connect via Driver " + driver + "; url ="
                    + url + " user=" + user);

            conn = DriverManager.getConnection(url, user, password);
        }
        catch ( Exception e )
        {
            log.error(e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return conn;
    }

    /**
     * Close a ResultSet.
     * 
     * @param rs
     *            a <code>ResultSet</code> value
     */
    public static void cleanup(ResultSet rs)
    {
        try
        {
            if ( rs != null )
            {
                rs.close();
            }
        }
        catch ( SQLException e )
        {
            log.warn(e.getMessage());
        }
    }

    /**
     * Close a Statement.
     * 
     * @param stmt
     */
    public static void cleanup(Statement stmt)
    {
        try
        {
            if ( stmt != null )
            {
                stmt.close();
            }
        }
        catch ( SQLException e )
        {
            log.warn(e.getMessage());
        }
    }

    /**
     * Close a Connection.
     * 
     * @param conn
     */
    public static void cleanup(Connection conn)
    {
        try
        {
            if ( conn != null )
            {
                conn.close();
            }
        }
        catch ( Exception e )
        {
            log.warn(e.getMessage());
        }
    }
}
