package com.itko.lisa.ext.util;

import java.io.File;
import java.io.InputStream;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang.StringUtils;

import com.itko.lisa.model.mar.ModelArchive;
import com.itko.lisa.model.mar.ModelArchiveInfo;
import com.itko.lisa.test.TestCase;
import com.itko.lisa.vse.stateful.model.ArgumentType;
import com.itko.lisa.vse.stateful.model.Conversation;
import com.itko.lisa.vse.stateful.model.Request;
import com.itko.lisa.vse.stateful.model.ServiceImage;
import com.itko.lisa.vse.stateful.model.ServiceImageCombiner;
import com.itko.lisa.vse.stateful.model.Transaction;
import com.itko.lisa.vse.stateful.model.TransactionNode;
import com.itko.lisa.vse.stateful.model.xml.ServiceImageContext;
import com.itko.lisa.vse.stateful.protocol.DataProtocol;
import com.itko.lisa.vse.stateful.protocol.TransportProtocol;
import com.itko.lisa.vse.stateful.protocol.rrpairs.VSFromRRPairsTransportProtocol;
import com.itko.lisa.vse.stateful.recorder.RecordingSession;
import com.itko.util.Parameter;
import com.itko.util.StrUtil;

/**
 * ServiceImageHelper - Helper for interacting with transaction data from
 * Service Images
 *
 * @author <a href="mailto:daniel.bingham@itko.com">Daniel Bingham</a>
 */
@Deprecated
public class ServiceImageHelper
{
    /** Field logger */
    private static final Logger logger = Logger.getLogger( ServiceImageHelper.class.getName() );

    public static ServiceImage loadServiceImage( final String siPath )
    {
        return ServiceImageContext.readServiceImage( siPath );
    }

    public static ServiceImage loadServiceImage( final File siFile )
    {
        try
        {
            return ServiceImageContext.readServiceImage( siFile.getCanonicalPath() );
        }
        catch( Exception e )
        {
            logger.log( Level.SEVERE, e.getMessage(), e );
            throw new RuntimeException( e.getMessage(), e );
        }
    }

    public static void saveServiceImage( final ServiceImage si, final String path )
    {
        ServiceImageContext.writeXML( path, si );
    }

    public static void renameServiceImage( final String siPath, final String newName )
    {
        ServiceImage si = loadServiceImage( siPath );

        if( si != null )
        {
            si.setName( newName );

            saveServiceImage( si, siPath );
        }
    }

    public static void combineServiceImages( final String sourceSi, final String targetSi, boolean favorSource )
    {
        ServiceImageCombiner siCombiner = new ServiceImageCombiner( targetSi, favorSource );
        siCombiner.combineWith( sourceSi );
        siCombiner.save();
    }

    public static ServiceImage combineServiceImages( final ServiceImage sourceSi,
                                                     final ServiceImage targetSi,
                                                     boolean favorSource )
    {
        try
        {
            File sourceTmp = File.createTempFile( "vsi-", ".vsi" );
            File targetTmp = File.createTempFile( "vsi-", ".vsi" );

            saveServiceImage( sourceSi, sourceTmp.getAbsolutePath() );
            saveServiceImage( targetSi, targetTmp.getAbsolutePath() );

            combineServiceImages( sourceTmp.getAbsolutePath(), targetTmp.getAbsolutePath(), favorSource );

            return ServiceImageContext.readServiceImage( targetTmp.getAbsolutePath() );
        }
        catch( Exception e )
        {
            logger.log( Level.SEVERE, e.getMessage(), e );
            throw new RuntimeException( e.getMessage(), e );
        }
    }

    public static ServiceImage getSIFromMar( ModelArchive mar )
    {
        ServiceImage si = null;

        try
        {
            String vsm = mar.getMARInfo().getDeploymentParameter( ModelArchiveInfo.PRIMARY_ASSET );
            InputStream vsmIS = mar.openEntry( vsm );
            TestCase vsmtc = TestCaseHelper.loadVsm( vsmIS );
            String siFile = TestCaseHelper.getSIPathFromVSM( vsmtc );

            String[] entries = mar.getEntryNames();
            for( String entry : entries )
            {
                if( siFile.endsWith( entry ) )
                {
                    InputStream siIS = mar.openEntry( entry );
                    si = ServiceImageContext.readServiceImage( siIS );
                    break;
                }
            }
        }
        catch( Exception e )
        {
            logger.log( Level.SEVERE, e.getMessage(), e );
            throw new RuntimeException( e.getMessage(), e );
        }

        return si;
    }

    public static void mixRawTrafficIntoSi( final String rawTrafficFile,
                                            final ServiceImage si,
                                            final String vrsFile )
    {
        RecordingSession session = new RecordingSession( new File( vrsFile ) );

        doMixRtIntoSi( rawTrafficFile, si, session );
    }

    public static void mixRawTrafficIntoSi( final String rawTrafficFile,
                                            final ServiceImage si,
                                            final TransportProtocol tph,
                                            final List<DataProtocol> reqDataProtocols,
                                            final List<DataProtocol> rspDataProtocols )
    {
        if( reqDataProtocols != null )
        {
            for( DataProtocol dph : reqDataProtocols )
            {
                tph.addDataProtocol( dph, true );
            }
        }

        if( rspDataProtocols != null )
        {
            for( DataProtocol dph : rspDataProtocols )
            {
                tph.addDataProtocol( dph, false );
            }
        }

        RecordingSession session = RecordingSession.createRecordingSession( si.getName(),
                                                                            tph,
                                                                            null );

        doMixRtIntoSi( rawTrafficFile, si, session );
    }

    private static void doMixRtIntoSi( final String rawTrafficFile,
                                       final ServiceImage si,
                                       final RecordingSession session )
    {
        session.setServiceImage( si );

//        session.importTransactions(rawTrafficFile);
        session.setImportFileName (rawTrafficFile);
        session.importTransactions ( );

        session.storeRecordedTransactions();
        session.complete();
    }

    /**
     * Make an SI from RRPairs
     *
     * @param rrPairsDir       - the RR Pairs dir
     * @param siName           - the name for the resultant SI
     * @param tph              - the Transport protocol to use for parsing the traffic
     * @param reqDataProtocols - the request side data protocols to use
     * @param rspDataProtocols - the response side data protocols to use
     *
     * @return a ServiceImage object if everything went well, or NULL if not
     */
    public static ServiceImage siFromRRPairs( final String rrPairsDir,
                                              final String siName,
                                              final TransportProtocol tph,
                                              final List<DataProtocol> reqDataProtocols,
                                              final List<DataProtocol> rspDataProtocols )
    {
        ServiceImage si;

        try
        {
            VSFromRRPairsTransportProtocol rrPairsTPH = getRRPairsProtocol( reqDataProtocols, rspDataProtocols );

            RecordingSession session = RecordingSession.createRecordingSession( siName, rrPairsTPH, null );

           // rrPairsTPH.configure( rrPairsDir, tph );
            session.storeRecordedTransactions();
            si = session.getServiceImage();
            session.complete();
        }
        catch( Exception e )
        {
            logger.log( Level.SEVERE, e.getMessage(), e );
            throw new RuntimeException( e.getMessage(), e );
        }

        return si;
    }

    /**
     * Get Raw Traffic from RR Pairs
     *
     * @param rrPairsDir - the RR pairs dir
     * @param rtPath     - the resultant RT
     * @param tph        - the TPH to use when parsing the RR pairs and generating the RT
     */
    public static void rtFromRRPairs( final String rrPairsDir,
                                      final String rtPath,
                                      final TransportProtocol tph )
    {
        try
        {
            VSFromRRPairsTransportProtocol rrPairsTPH = getRRPairsProtocol( null, null );

//          rrPairsTPH.getLineJudge().setExportFileName( rtPath );
          rrPairsTPH.setExportFileName( rtPath );

           // rrPairsTPH.configure( rrPairsDir, tph );
        }
        catch( Exception e )
        {
            logger.log( Level.SEVERE, e.getMessage(), e );
            throw new RuntimeException( e.getMessage(), e );
        }
    }

    private static VSFromRRPairsTransportProtocol getRRPairsProtocol( final List<DataProtocol> reqDataProtocols,
                                                                      final List<DataProtocol> rspDataProtocols )
    {
        VSFromRRPairsTransportProtocol tph = new VSFromRRPairsTransportProtocol();

        if( reqDataProtocols != null )
        {
            for( DataProtocol dph : reqDataProtocols )
            {
                tph.addDataProtocol( dph, true );
            }
        }

        if( rspDataProtocols != null )
        {
            for( DataProtocol dph : rspDataProtocols )
            {
                tph.addDataProtocol( dph, false );
            }
        }

        return tph;
    }

    /**
     * Method setArgumentMatchOperator replaces the current match operator on
     * the specified operation, for the given arguments, with the specified
     * operator.
     *
     * @param serviceImage   of type Traffic - The service image to modify
     * @param operationNames of type String[] - The list of operations to work against
     * @param arguments      of type String[] - The list of arguments to be changed
     * @param operator       of type String - The operator to use
     */
    public static void setArgumentMatchOperator( final ServiceImage serviceImage,
                                                 final String[] operationNames,
                                                 final String[] arguments,
                                                 final String operator )
    {
        for( String operation : operationNames )
        {
            setArgumentMatchOperator( serviceImage,
                                      operation,
                                      arguments,
                                      operator );
        }
    }

    /**
     * Method setArgumentMatchOperator replaces the current match operator on an
     * operation, for the given arguments, with the specified operator.
     *
     * @param serviceImage  of type Traffic - The service image to modify
     * @param operationName of type String - The operation name to work against
     * @param arguments     of type String[] - The list of arguments to be changed
     * @param operator      of type String - The operator to use
     */
    @SuppressWarnings( { "TypeMayBeWeakened" } )
    public static void setArgumentMatchOperator( final ServiceImage serviceImage,
                                                 final String operationName,
                                                 final String[] arguments,
                                                 final String operator )
    {
        try
        {
            // Start with the stateless transactions
            for( TransactionNode trans : serviceImage.getStatelessTransactions() )
            {
                setMatchOpOnSpecifics( trans,
                                       operationName,
                                       arguments,
                                       operator );
            }

            for( Conversation conv : serviceImage.getConversations() )
            {
                TransactionNode starter = conv.getStarter();
                setMatchOpOnTransaction( starter,
                                         operationName,
                                         arguments,
                                         operator );
            }
        }
        catch( Exception e )
        {
            logger.log( Level.SEVERE, e.getMessage(), e );
            throw new RuntimeException( e.getMessage(), e );
        }
    }

    /**
     * Method setMatchOpOnTransaction recurses through the children of the
     * transaction and processes the specifics from each as it goes.
     *
     * @param trans         of type Transaction - The root transaction of the recurrence
     * @param operationName of type String - The name of the operation we're looking for
     * @param arguments     of type String[] - The arguments we are looking for on the
     *                      request
     * @param operator      of type String - The desired match operator for the arguments
     */
    private static void setMatchOpOnTransaction( final TransactionNode trans,
                                                 final String operationName,
                                                 final String[] arguments,
                                                 final String operator )
    {
        // Recurse down over the children, Base Condition: no children
        for( TransactionNode child : trans.getChildren() )
        {
            setMatchOpOnTransaction( child,
                                     operationName,
                                     arguments,
                                     operator );
        }

        // Do the actual work on the transaction at hand
        setMatchOpOnSpecifics( trans, operationName, arguments, operator );

    }

    /**
     * Method setMatchOpOnSpecifics checks to see if the transaction is for an
     * operation that we care about. If it is, it iterates through the specifics
     * and modifies the arguments to set them to use the specified match
     * operator
     *
     * @param meta          of type Transaction - The meta transaction for the operation
     *                      in question
     * @param operationName of type String - The name of the operation we're looking for
     * @param arguments     of type String[] - The arguments to change on the request
     * @param operator      of type String - The desired match operator for the arguments
     */
    private static void setMatchOpOnSpecifics( final TransactionNode meta,
                                               final String operationName,
                                               final String[] arguments,
                                               final String operator )
    {
        // Get the request object and see if it is for an operation we're
        // interested in
        Request req = meta.getRequest();
        if( StringUtils.equalsIgnoreCase( req.getOperation(), operationName )
            || StringUtils.equals( operationName, "*" ) )
        {
            // Grab the specifics, to modify their arguments
            for( Transaction specific : meta.getSpecifics() )
            {
                // Get the request off the specific
                Request spReq = specific.getRequest();

                // Get the arguments for the request
                for( Parameter param : spReq.getArguments() )
                {
                    // If this argument is one that needs to be changed...
                    if( StrUtil.isStringInStrings( param.getKey(),
                                                   arguments,
                                                   false )
                        || StrUtil.isStringInStrings( "*", arguments, false ) )
                    {
                        // Change the operator to the one they requested
                        ArgumentType op = (ArgumentType) param.getType();
                        op.setOperator( ArgumentType.Operator.valueOf( operator ) );
                    }
                }
            }
        }
    }
}
