package com.itko.lisa.ext.util;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang.StringUtils;

import com.itko.lisa.ext.exception.MissingParameterException;
import com.itko.lisa.test.TestEvent;
import com.itko.lisa.utils.ErrorReportGenerator;

/**
 * Generate an HTML test report using the <code>ErrorReportGenerator</code>.
 * 
 * @author <a href="mailto:luther@itko.com">Luther Birdzell</a>
 */
public class TestReportHelper
{
    private static Logger log                  = Logger.getLogger(TestReportHelper.class.getName());

    /**
     * 
     */
    public static String  TEST_REPORT_XSL_PATH = "com/itko/lisa/gui/xsl/error-report.xsl";

    /**
     * 
     * 
     * @param testEvent
     * @return the HTML report as a <code>String</code>.
     */
    public static String generateTestReport(TestEvent testEvent)
    {
        return generateTestReport(TEST_REPORT_XSL_PATH, testEvent);
    }

    /**
     * 
     * 
     * @param xslPath
     * @param testEvent
     * @return the HTML report as a <code>String</code>.
     */
    public static String generateTestReport(String xslPath, TestEvent testEvent)
    {
        String report = null;

        try
        {
            // generate a test report
            if ( testEvent != null )
            {
                log.fine("testEventXml: " + testEvent.getLongDesc());

                ErrorReportGenerator generator = new ErrorReportGenerator(xslPath);

                // copy the report under the lisa-invoke webapp
                // so we can serve it externally
                String reportPath = generator.getReport(testEvent);
                log.info("reportPath = " + reportPath);

                if ( StringUtils.isNotBlank(reportPath) )
                {
                    // get the HTML report as a String
                    report = FileHelper.getFileAsString(reportPath);
                }
            }
        }
        catch ( Exception e )
        {
            log.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return report;
    }

    /**
     * 
     * 
     * @param xslPath
     * @param testEvent
     * @param outputDir
     * @param testCaseName
     * @param serverReportUrl
     * @return the HTML report name
     */
    public static String generateTestReport(String xslPath,
                                            TestEvent testEvent,
                                            String outputDir,
                                            String testCaseName,
                                            String serverReportUrl)
    {
        String reportName = null;

        if ( testEvent == null || StringUtils.isBlank(outputDir)
                || StringUtils.isBlank(testCaseName) )
        {
            throw new MissingParameterException("testEvent, outputDir and testCaseName");
        }

        try
        {
            String report = generateTestReport(xslPath, testEvent);

            // update the img references to point back to the server
            report = StringUtils.replace(report, "./img/", serverReportUrl
                    + "img/");

            reportName = getReportName(testCaseName);

            FileHelper.writeStringToFileSystem(report,
                                               FileHelper.ensureTrailingFileSeparator(outputDir)
                                                       + reportName);
        }
        catch ( Exception e )
        {
            log.log(Level.SEVERE, e.getMessage(), e);
            throw new RuntimeException(e.getMessage(), e);
        }

        return reportName;
    }

    /**
     * 
     * 
     * @param testCasePath
     * @return the name of the test report
     */
    public static String getReportName(String testCasePath)
    {
        String testCaseName = FileHelper.getFileName(testCasePath);

        String reportName = testCaseName + "-" + System.currentTimeMillis()
                + ".html";

        return reportName;
    }

    /**
     * 
     * 
     * @param testCasePath
     * @param callbackKey
     * 
     * @return the name of the test report
     */
    public static String getReportName(String testCasePath, String callbackKey)
    {
        String testCaseName = FileHelper.getFileName(testCasePath);

        String reportName = testCaseName + "-" + callbackKey + ".html";

        return reportName;
    }
}
