package com.itko.lisa.ext.util.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.TestCase;

import org.apache.commons.io.IOUtils;

/**
 * Process used to prevent a test from running more than once in a specified
 * time interval using a filesystem mutex.
 * 
 * @author <a href="mailto:luther@itko.com">Luther Birdzell</a>
 */
public class FileSystemMutexTest
        extends TestCase
{
    private static Logger       log                            = Logger.getLogger(FileSystemMutexTest.class.getName());

    private static final int    SECONDS_TO_WAIT_UNTIL_NEXT_RUN = 50;
    private static final String TEST_CASE_NAME                 = "MyTest";

    /**
     * 
     * 
     */
    public void testCreateLockFile()
    {
        try
        {
            File lockFile = new File(TEST_CASE_NAME);
            int secondsToWaitUntilNextRun = SECONDS_TO_WAIT_UNTIL_NEXT_RUN;

            if ( lockFile.exists() )
            {
                FileInputStream input = new FileInputStream(lockFile);
                List < String > lines = IOUtils.readLines(input);
                IOUtils.closeQuietly(input);

                String timeOfLastTestString = lines.get(0);
                long timeOfLastTest = Long.parseLong(timeOfLastTestString);
                long currentTime = System.currentTimeMillis();
                log.fine("timeOfLastTest = " + new Date(timeOfLastTest)
                        + ", currentTime = " + new Date(currentTime));

                // if the test ran less than 1 minute ago
                if ( currentTime - timeOfLastTest > (secondsToWaitUntilNextRun * 1000) )
                {
                    log.info("the test ran over " + secondsToWaitUntilNextRun
                            + "  seconds ago, running test");

                    FileOutputStream output = new FileOutputStream(lockFile);
                    IOUtils.write(String.valueOf(System.currentTimeMillis()),
                                  output);
                    IOUtils.closeQuietly(output);
                }
                else
                {
                    log.info("the test already ran within the "
                            + secondsToWaitUntilNextRun
                            + " second window, not running...");
                }
            }
            else
            {
                log.info("lockfile did not exist running test");
                FileOutputStream output = new FileOutputStream(lockFile);
                IOUtils.write(String.valueOf(System.currentTimeMillis()),
                              output);
                IOUtils.closeQuietly(output);
            }
        }
        catch ( Throwable e )
        {
            log.log(Level.SEVERE, e.getMessage(), e);
            fail(e.getMessage());
        }
    }
}
